@@ -14,11 +14,15 @@
 package api
 
 import (
+	"os"
+	"strconv"
+
 	"github.com/aws/aws-sdk-go/aws"
 	"github.com/aws/aws-sdk-go/aws/endpoints"
 	"github.com/aws/aws-sdk-go/aws/request"
 	"github.com/aws/aws-sdk-go/aws/session"
 	"github.com/aws/aws-sdk-go/service/ecr"
+	"github.com/aws/aws-sdk-go/service/ecrpublic"
 	"github.com/awslabs/amazon-ecr-credential-helper/ecr-login/cache"
 	"github.com/awslabs/amazon-ecr-credential-helper/ecr-login/version"
 )
@@ -49,7 +53,10 @@ var userAgentHandler = request.NamedHandler{
 
 // NewClientWithDefaults creates the client and defaults region
 func (defaultClientFactory DefaultClientFactory) NewClientWithDefaults() Client {
-	awsSession := session.New()
+	awsSession := session.Must(session.NewSessionWithOptions(session.Options{
+		SharedConfigState: loadSharedConfigState(),
+	}))
+
 	awsSession.Handlers.Build.PushBackNamed(userAgentHandler)
 	awsConfig := awsSession.Config
 	return defaultClientFactory.NewClientWithOptions(Options{
@@ -60,7 +67,10 @@ func (defaultClientFactory DefaultClientFactory) NewClientWithDefaults() Client
 
 // NewClientWithFipsEndpoint overrides the default ECR service endpoint in a given region to use the FIPS endpoint
 func (defaultClientFactory DefaultClientFactory) NewClientWithFipsEndpoint(region string) (Client, error) {
-	awsSession := session.New()
+	awsSession := session.Must(session.NewSessionWithOptions(session.Options{
+		SharedConfigState: loadSharedConfigState(),
+	}))
+
 	awsSession.Handlers.Build.PushBackNamed(userAgentHandler)
 
 	endpoint, err := getServiceEndpoint("ecr-fips", region)
@@ -77,7 +87,9 @@ func (defaultClientFactory DefaultClientFactory) NewClientWithFipsEndpoint(regio
 
 // NewClientFromRegion uses the region to create the client
 func (defaultClientFactory DefaultClientFactory) NewClientFromRegion(region string) Client {
-	awsSession := session.New()
+	awsSession := session.Must(session.NewSessionWithOptions(session.Options{
+		SharedConfigState: loadSharedConfigState(),
+	}))
 	awsSession.Handlers.Build.PushBackNamed(userAgentHandler)
 	awsConfig := &aws.Config{Region: aws.String(region)}
 	return defaultClientFactory.NewClientWithOptions(Options{
@@ -96,8 +108,11 @@ func (defaultClientFactory DefaultClientFactory) NewClient(awsSession *session.S
 
 // NewClientWithOptions Create new client with Options
 func (defaultClientFactory DefaultClientFactory) NewClientWithOptions(opts Options) Client {
+	// The ECR Public API is only available in us-east-1 today
+	publicConfig := opts.Config.Copy().WithRegion("us-east-1")
 	return &defaultClient{
 		ecrClient:       ecr.New(opts.Session, opts.Config),
+		ecrPublicClient: ecrpublic.New(opts.Session, publicConfig),
 		credentialCache: cache.BuildCredentialsCache(opts.Session, aws.StringValue(opts.Config.Region), opts.CacheDir),
 	}
 }
@@ -109,3 +124,14 @@ func getServiceEndpoint(service, region string) (string, error) {
 	})
 	return endpoint.URL, err
 }
+
+func loadSharedConfigState() session.SharedConfigState {
+	loadConfig := os.Getenv("AWS_SDK_LOAD_CONFIG")
+	if loadConfig == "" {
+		return session.SharedConfigEnable
+	}
+	if enable, err := strconv.ParseBool(loadConfig); err == nil && !enable {
+		return session.SharedConfigDisable
+	}
+	return session.SharedConfigEnable
+}
